<?php
/* --------------------------------------------------------------
  ContentReader.php 2019-07-30
  Gambio GmbH
  http://www.gambio.de
  Copyright (c) 2019 Gambio GmbH
  Released under the GNU General Public License (Version 2)
  [http://www.gnu.org/licenses/gpl-2.0.html]
  --------------------------------------------------------------*/

/**
 * Class ContentReader
 */
class ContentReader implements ContentReaderInterface
{
    /**
     * CodeIgniter QueryBuilder
     *
     * @var CI_DB_query_builder
     */
    protected $queryBuilder;
    
    
    /**
     * ContentReader constructor.
     *
     * @param CI_DB_query_builder $queryBuilder
     */
    public function __construct(CI_DB_query_builder $queryBuilder)
    {
        $this->queryBuilder = $queryBuilder;
    }
    
    
    /**
     * @param mixed $id id of the Content
     *
     * @return array
     * @throws ContentNotFoundException
     */
    public function findById($id): array
    {
        $result = $this->queryBuilder->select('*')
            ->from('content_manager')
            ->where('content_group', $id)
            ->get()
            ->result_array();
        
        if (count($result) === 0) {
            
            throw new ContentNotFoundException($id);
        }
        
        return $result;
    }
    
    
    /**
     * @param $id
     *
     * @return array
     * @throws UrlRewriteNotFoundException
     */
    public function findUrlRewriteByContentId($id): array
    {
        $result = $this->queryBuilder->select('*')
            ->from('url_rewrites')
            ->where('content_id', $id)
            ->get()
            ->result_array();
        
        if (count($result) === 0) {
            
            throw new UrlRewriteNotFoundException($id);
        }
        
        return $result;
    }
    
    
    /**
     * @return int
     */
    public function nextContentGroupId(): int
    {
        $result = $this->queryBuilder->select('MAX(content_group)')->from('content_manager')->get()->result_array();
        $result = (int)$result[0]['MAX(content_group)'];
        
        return ++$result;
    }
    
    
    /**
     * @return array
     * @throws ContentNotFoundException
     */
    public function findAllInfoElements(): array
    {
        $result = $this->queryBuilder->select()
            ->from('content_manager')
            ->like('content_position', 'elements', 'after')
            ->order_by('content_group')
            ->get()
            ->result_array();
        
        if (count($result) === 0) {
            
            throw new ContentNotFoundException('*');
        }
        
        return $result;
    }
}