<?php
/* --------------------------------------------------------------
   BasicElementController.inc.php 2019-10-14
   Gambio GmbH
   http://www.gambio.de
   Copyright (c) 2018 Gambio GmbH
   Released under the GNU General Public License (Version 2)
   [http://www.gnu.org/licenses/gpl-2.0.html]
   --------------------------------------------------------------
*/

namespace Gambio\StyleEdit\Core\Components;

use Exception;
use Gambio\StyleEdit\Core\Components\Theme\Entities\Interfaces\CurrentThemeInterface;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfiguration;
use Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection;
use Gambio\StyleEdit\Core\Components\Theme\PreviewSettingsService;
use Gambio\StyleEdit\Core\Components\Theme\StyleEditThemeService;
use Gambio\StyleEdit\Core\SingletonPrototype;
use ReflectionException;

/**
 * Class BasicElementController
 * @package Gambio\StyleEdit\Core\Elements
 */
abstract class BasicController
{
    
    /**
     * @var bool
     */
    protected $previewSettingsService;
    /**
     * @var null | \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     */
    protected $themesList;
    /**
     * @var bool | ThemeConfiguration
     */
    private $theme = false;
    /**
     * @var null | \Gambio\StyleEdit\Core\Components\Theme\StyleEditThemeService
     */
    private $themeService;
    
    
    /**
     * @param array $uri
     *
     * @return mixed
     */
    abstract public function get(array $uri);
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed
     */
    abstract public function put(array $uri, $data);
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed
     */
    abstract public function post(array $uri, $data);
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed
     */
    abstract public function delete(array $uri, $data);
    
    
    /**
     * @param array $uri
     * @param       $data
     *
     * @return mixed
     */
    abstract public function patch(array $uri, $data);
    
    
    /**
     * @return BasicController
     */
    abstract public function __clone();
    
    
    /**
     * @return bool
     */
    public function requiresAuthentication(): bool
    {
        return true;
    }
    
    
    protected function updateThemeList()
    {
        
        $this->themeService()->updateThemeList();
        $this->themesList = null;
    }
    
    
    /**
     * @return bool|\Gambio\StyleEdit\Core\Components\Theme\StyleEditThemeService
     * @throws Exception
     */
    protected function themeService()
    {
        if ($this->themeService === null) {
            $this->themeService = SingletonPrototype::instance()->get(StyleEditThemeService::class);
            if (!$this->themeService) {
                throw new Exception('ThemeService not published');
            }
        }
        
        return $this->themeService;
    }
    
    
    /**
     * @return bool|PreviewSettingsService|mixed|null
     * @throws ReflectionException
     */
    protected function previewSettingsService()
    {
        if ($this->previewSettingsService === null) {
            $this->previewSettingsService = SingletonPrototype::instance()->get(PreviewSettingsService::class);
            if (!$this->previewSettingsService) {
                throw new Exception('PreviewSettingsService not published');
            }
        }
        
        return $this->previewSettingsService;
    }
    
    
    /**
     * @param ThemeConfiguration $value
     */
    protected function outputJson($value): void
    {
        header('Content-type: application/json');
        echo json_encode($value);
    }
    
    
    /**
     * @return bool | CurrentThemeInterface
     * @throws Exception
     */
    protected function currentTheme(): ?CurrentThemeInterface
    {
        if (!$this->theme) {
            $this->theme = SingletonPrototype::instance()->get(CurrentThemeInterface::class);
        }
        
        return $this->theme ? : null;
    }
    
    
    /**
     * @return string
     * @throws Exception
     */
    protected function currentThemeId() : string {
        if($this->currentTheme()){
            return $this->currentTheme()->id();
        }
        throw new Exception ('Theme ID is invalid or not supplied');
    }
    
    
    /**
     * @return \Gambio\StyleEdit\Core\Components\Theme\Entities\ThemeConfigurationCollection
     * @throws Exception
     */
    protected function getThemesList()
    {
        if ($this->themesList === null) {
            $this->themesList = SingletonPrototype::instance()->get(ThemeConfigurationCollection::class);
        }
        
        return $this->themesList;
    }
    
}